@@ -319,57 +319,14 @@ bool ForecastMachine::is_target_valid(const size_t vec_index)
     return true;
 }
 
-PredStats ForecastMachine::compute_stats(const vec& obs, const vec& pred)
-{
-    size_t num_pred = 0;
-    double sum_errors = 0;
-    double sum_squared_errors = 0;
-    double sum_obs = 0;
-    double sum_pred = 0;
-    double sum_squared_obs = 0;
-    double sum_squared_pred = 0;
-    double sum_prod = 0;
-    size_t same_sign = 0;
-    
-    for(size_t k = 0; k < num_vectors; ++k)
-    {
-        if(!std::isnan(obs[k]) && !std::isnan(pred[k]))
-        {
-            ++ num_pred;
-            sum_errors += fabs(obs[k] - pred[k]);
-            sum_squared_errors += (obs[k] - pred[k]) * (obs[k] - pred[k]);
-            sum_obs += obs[k];
-            sum_pred += pred[k];
-            sum_squared_obs += obs[k] * obs[k];
-            sum_squared_pred += pred[k] * pred[k];
-            sum_prod += obs[k] * pred[k];
-            if((obs[k] >= 0 && pred[k] >= 0) ||
-               (obs[k] <= 0 && pred[k] <= 0))
-               ++ same_sign;
-        }
-    }
-    
-    PredStats output;
-    output.num_pred = num_pred;
-    output.rho = (sum_prod * num_pred - sum_obs * sum_pred) /
-    sqrt((sum_squared_obs * num_pred - sum_obs * sum_obs) *
-         (sum_squared_pred * num_pred - sum_pred * sum_pred));
-    output.mae = sum_errors / double(num_pred);
-    output.rmse = sqrt(sum_squared_errors / double(num_pred));
-    output.perc = double(same_sign) / double(num_pred);
-    output.p_val = R::pnorm(atanh(output.rho), 0.0, 1.0 / sqrt(double(output.num_pred-3)), 0.0, 0.0);
-
-    return output;
-}
-
 PredStats ForecastMachine::make_stats()
 {
-    return compute_stats(targets, predicted);
+    return compute_stats_internal(targets, predicted);
 }
 
 PredStats ForecastMachine::make_const_stats()
 {
-    return compute_stats(targets, const_predicted);
+    return compute_stats_internal(targets, const_predicted);
 }
 
 void ForecastMachine::LOG_WARNING(const char* warning_text)
@@ -605,7 +562,7 @@ void ForecastMachine::smap_prediction(const size_t start, const size_t end)
         }
         
         // perform SVD
-        JacobiSVD<MatrixXd> svd(A, ComputeThinU | ComputeThinV);
+        Eigen::JacobiSVD<MatrixXd> svd(A, Eigen::ComputeThinU | Eigen::ComputeThinV);
         
         // remove singular values close to 0
         S = svd.singularValues();
@@ -658,7 +615,7 @@ void ForecastMachine::adjust_lib(const size_t curr_pred)
     }
     else
     {
-        which_lib.erase(std::remove_if(which_lib.begin(), which_lib.end(), [&](const size_t curr_lib) {return curr_lib == curr_pred;}), which_lib.end());
+        which_lib.erase(std::remove(which_lib.begin(), which_lib.end(), curr_pred), which_lib.end());
     }
     return;
 }
@@ -682,3 +639,62 @@ std::vector<size_t> sort_indices(const vec& v, std::vector<size_t> idx)
          [&v](size_t i1, size_t i2) {return v[i1] < v[i2];});
     return idx;
 }
+
+PredStats compute_stats_internal(const vec& obs, const vec& pred)
+{
+    size_t num_pred = 0;
+    double sum_errors = 0;
+    double sum_squared_errors = 0;
+    double sum_obs = 0;
+    double sum_pred = 0;
+    double sum_squared_obs = 0;
+    double sum_squared_pred = 0;
+    double sum_prod = 0;
+    size_t same_sign = 0;
+    size_t num_vectors = obs.size();
+    if(pred.size() < num_vectors)
+        num_vectors = pred.size();
+    
+    for(size_t k = 0; k < num_vectors; ++k)
+    {
+        if(!std::isnan(obs[k]) && !std::isnan(pred[k]))
+        {
+            ++ num_pred;
+            sum_errors += fabs(obs[k] - pred[k]);
+            sum_squared_errors += (obs[k] - pred[k]) * (obs[k] - pred[k]);
+            sum_obs += obs[k];
+            sum_pred += pred[k];
+            sum_squared_obs += obs[k] * obs[k];
+            sum_squared_pred += pred[k] * pred[k];
+            sum_prod += obs[k] * pred[k];
+            if((obs[k] >= 0 && pred[k] >= 0) ||
+               (obs[k] <= 0 && pred[k] <= 0))
+                ++ same_sign;
+        }
+    }
+    
+    PredStats output;
+    output.num_pred = num_pred;
+    output.rho = (sum_prod * num_pred - sum_obs * sum_pred) /
+    sqrt((sum_squared_obs * num_pred - sum_obs * sum_obs) *
+        (sum_squared_pred * num_pred - sum_pred * sum_pred));
+    output.mae = sum_errors / double(num_pred);
+    output.rmse = sqrt(sum_squared_errors / double(num_pred));
+    output.perc = double(same_sign) / double(num_pred);
+    output.p_val = R::pnorm(atanh(output.rho), 0.0, 1.0 / sqrt(double(output.num_pred-3)), 0.0, 0.0);
+    
+    return output;
+}
+
+// [[Rcpp::export]]
+DataFrame compute_stats(std::vector<double> observed, std::vector<double> predicted)
+{
+
+    PredStats output = compute_stats_internal(observed, predicted);
+    return DataFrame::create( Named("num_pred") = output.num_pred, 
+                              Named("rho") = output.rho, 
+                              Named("mae") = output.mae, 
+                              Named("rmse") = output.rmse,
+                              Named("perc") = output.perc, 
+                              Named("p_val") = output.p_val);
+}
